@@ -1,7 +1,6 @@
 typedef union {
     CSSRuleImpl *rule;
     CSSSelector *selector;
-    QPtrList<CSSSelector> *selectorList;
     bool ok;
     MediaListImpl *mediaList;
     CSSMediaRuleImpl *mediaRule;
@@ -17,7 +16,7 @@ typedef union {
     Value value;
     ValueList *valueList;
 } YYSTYPE;
-#define	S	257
+#define	WHITESPACE	257
 #define	SGML_CD	258
 #define	INCLUDES	259
 #define	DASHMATCH	260
@@ -32,30 +31,31 @@ typedef union {
 #define	MEDIA_SYM	269
 #define	FONT_FACE_SYM	270
 #define	CHARSET_SYM	271
-#define	KHTML_RULE_SYM	272
-#define	KHTML_DECLS_SYM	273
-#define	KHTML_VALUE_SYM	274
-#define	IMPORTANT_SYM	275
-#define	QEMS	276
-#define	EMS	277
-#define	EXS	278
-#define	PXS	279
-#define	CMS	280
-#define	MMS	281
-#define	INS	282
-#define	PTS	283
-#define	PCS	284
-#define	DEGS	285
-#define	RADS	286
-#define	GRADS	287
-#define	MSECS	288
-#define	SECS	289
-#define	HERZ	290
-#define	KHERZ	291
-#define	DIMEN	292
-#define	PERCENTAGE	293
-#define	NUMBER	294
-#define	URI	295
-#define	FUNCTION	296
-#define	UNICODERANGE	297
+#define	NAMESPACE_SYM	272
+#define	KHTML_RULE_SYM	273
+#define	KHTML_DECLS_SYM	274
+#define	KHTML_VALUE_SYM	275
+#define	IMPORTANT_SYM	276
+#define	QEMS	277
+#define	EMS	278
+#define	EXS	279
+#define	PXS	280
+#define	CMS	281
+#define	MMS	282
+#define	INS	283
+#define	PTS	284
+#define	PCS	285
+#define	DEGS	286
+#define	RADS	287
+#define	GRADS	288
+#define	MSECS	289
+#define	SECS	290
+#define	HERZ	291
+#define	KHERZ	292
+#define	DIMEN	293
+#define	PERCENTAGE	294
+#define	NUMBER	295
+#define	URI	296
+#define	FUNCTION	297
+#define	UNICODERANGE	298
 
