@@ -21,6 +21,7 @@
 
 #include "kjs_binding.h"
 #include "kjs_dom.h"
+#include "kjs_window.h"
 #include <kjs/internal.h> // for InterpreterImp
 
 #include "dom/dom_exception.h"
@@ -273,14 +274,52 @@ bool ScriptInterpreter::wasRunByUserGesture() const
   return false;
 }
 
+#if APPLE_CHANGES
+bool ScriptInterpreter::isGlobalObject(const Value &v)
+{
+    if (v.type() == ObjectType) {
+	Object o = v.toObject (globalExec());
+	if (o.classInfo() == &Window::info)
+	    return true;
+    }
+    return false;
+}
+
+bool ScriptInterpreter::isSafeScript (const Interpreter *_target)
+{
+    const KJS::ScriptInterpreter *target = static_cast<const ScriptInterpreter *>(_target);
+
+    return KJS::Window::isSafeScript (this, target);
+}
+
+Interpreter *ScriptInterpreter::interpreterForGlobalObject (const ValueImp *imp)
+{
+    const KJS::Window *win = static_cast<const KJS::Window *>(imp);
+    return win->interpreter();
+}
+
+void *ScriptInterpreter::createLanguageInstanceForValue (ExecState *exec, Bindings::Instance::BindingLanguage language, const Object &value, const Bindings::RootObject *origin, const Bindings::RootObject *current)
+{
+    void *result = 0;
+    
+    if (language == Bindings::Instance::ObjectiveCLanguage)
+	result = createObjcInstanceForValue (exec, value, origin, current);
+    
+    if (!result)
+	result = Interpreter::createLanguageInstanceForValue (exec, language, value, origin, current);
+	
+    return result;
+}
+
+#endif
+
 //////
 
 UString::UString(const QString &d)
 {
-  unsigned int len = d.length();
-  UChar *dat = new UChar[len];
-  memcpy(dat, d.unicode(), len * sizeof(UChar));
-  rep = UString::Rep::create(dat, len);
+  // reinterpret_cast is ugly but in this case safe, since QChar and UChar have the same
+  // memory layout
+  rep = UString::Rep::createCopying(reinterpret_cast<const UChar *>(d.unicode()), d.length());
 }
 
 UString::UString(const DOMString &d)
@@ -289,11 +328,9 @@ UString::UString(const DOMString &d)
     attach(&Rep::null);
     return;
   }
-
-  unsigned int len = d.length();
-  UChar *dat = new UChar[len];
-  memcpy(dat, d.unicode(), len * sizeof(UChar));
-  rep = UString::Rep::create(dat, len);
+  // reinterpret_cast is ugly but in this case safe, since QChar and UChar have the same
+  // memory layout
+  rep = UString::Rep::createCopying(reinterpret_cast<const UChar *>(d.unicode()), d.length());
 }
 
 DOMString UString::string() const
