@@ -40,7 +40,7 @@ public:
   virtual ~KJSProxyImpl();
   virtual QVariant evaluate(QString filename, int baseLine, const QString&str, const DOM::Node &n);
   virtual void clear();
-  virtual DOM::EventListener *createHTMLEventHandler(QString sourceUrl, QString code);
+  virtual DOM::EventListener *createHTMLEventHandler(QString sourceUrl, QString code, DOM::NodeImpl *node);
   virtual void finishedWithEvent(const DOM::Event &event);
   virtual KJS::ScriptInterpreter *interpreter();
 
@@ -114,7 +114,10 @@ QVariant KJSProxyImpl::evaluate(QString filename, int baseLine,
   m_script->setInlineCode(inlineCode);
   KJS::Value thisNode = n.isNull() ? Window::retrieve( m_part ) : getDOMNode(m_script->globalExec(),n);
 
+  KJS::Interpreter::lock();
   UString code( str );
+  KJS::Interpreter::unlock();
+
   Completion comp = m_script->evaluate(filename, baseLine, code, thisNode);
   bool success = ( comp.complType() == Normal ) || ( comp.complType() == ReturnValue );  
 #ifdef KJS_DEBUGGER
@@ -162,7 +165,7 @@ void KJSProxyImpl::clear() {
   }
 }
 
-DOM::EventListener *KJSProxyImpl::createHTMLEventHandler(QString sourceUrl, QString code)
+DOM::EventListener *KJSProxyImpl::createHTMLEventHandler(QString sourceUrl, QString code, DOM::NodeImpl *node)
 {
 #ifdef KJS_DEBUGGER
   if (KJSDebugWin::instance())
@@ -172,8 +175,7 @@ DOM::EventListener *KJSProxyImpl::createHTMLEventHandler(QString sourceUrl, QStr
 #endif
 
   initScript();
-
-  return KJS::Window::retrieveWindow(m_part)->getJSLazyEventListener(code,true);
+  return KJS::Window::retrieveWindow(m_part)->getJSLazyEventListener(code,node,m_handlerLineno);
 }
 
 void KJSProxyImpl::finishedWithEvent(const DOM::Event &event)
@@ -189,6 +191,7 @@ KJS::ScriptInterpreter *KJSProxyImpl::interpreter()
 {
   if (!m_script)
     initScript();
+  m_part->keepAlive();
   return m_script;
 }
 
