@@ -39,9 +39,11 @@ namespace KJS {
   class WindowQObject;
   class Location;
   class Selection;
+  class BarInfo;
   class History;
   class FrameArray;
   class JSEventListener;
+  class JSUnprotectedEventListener;
   class JSLazyEventListener;
 
   class Screen : public ObjectImp {
@@ -86,8 +88,8 @@ namespace KJS {
     static Window *retrieveActive(ExecState *exec);
     QGuardedPtr<KHTMLPart> part() const { return m_part; }
     virtual void mark();
-    virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
     virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual bool hasProperty(ExecState *exec, const Identifier &propertyName) const;
     virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
     virtual bool toBoolean(ExecState *exec) const;
     int installTimeout(const UString &handler, int t, bool singleShot);
@@ -97,13 +99,25 @@ namespace KJS {
     bool hasTimeouts();
     QMap<int, ScheduledAction*> *pauseTimeouts(const void *key);
     void resumeTimeouts(QMap<int, ScheduledAction*>*sa, const void *key);
+    
+    KJS::Interpreter *interpreter() const;
+
+    static bool isSafeScript (const KJS::ScriptInterpreter *origin, const KJS::ScriptInterpreter *target);
 #endif
     void scheduleClose();
+        
     bool isSafeScript(ExecState *exec) const;
     Location *location() const;
     Selection *selection() const;
+    BarInfo *locationbar(ExecState *exec) const;
+    BarInfo *menubar(ExecState *exec) const;
+    BarInfo *personalbar(ExecState *exec) const;
+    BarInfo *scrollbars(ExecState *exec) const;
+    BarInfo *statusbar(ExecState *exec) const;
+    BarInfo *toolbar(ExecState *exec) const;
     JSEventListener *getJSEventListener(const Value &val, bool html = false);
-    JSLazyEventListener *getJSLazyEventListener(const QString &code, bool html = false);
+    JSUnprotectedEventListener *getJSUnprotectedEventListener(const Value &val, bool html = false);
+    JSLazyEventListener *getJSLazyEventListener(const QString &code, DOM::NodeImpl *node, int lineno = 0);
     void clear( ExecState *exec );
     virtual UString toString(ExecState *exec) const;
 
@@ -111,12 +125,13 @@ namespace KJS {
     void setCurrentEvent( DOM::Event *evt );
 
     QPtrDict<JSEventListener> jsEventListeners;
+    QPtrDict<JSUnprotectedEventListener> jsUnprotectedEventListeners;
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
     enum { Closed, Crypto, DefaultStatus, Status, Document, Node, EventCtor, Range,
            NodeFilter, DOMException, CSSRule, Frames, _History, Event, InnerHeight,
-           InnerWidth, Length, _Location, Name, _Navigator, _Konqueror, ClientInformation,
-           OffscreenBuffering, Opener, OuterHeight, OuterWidth, PageXOffset, PageYOffset,
+           InnerWidth, Length, _Location, Locationbar, Name, _Navigator, _Konqueror, ClientInformation,
+           Menubar, OffscreenBuffering, Opener, OuterHeight, OuterWidth, PageXOffset, PageYOffset,
            Parent, Personalbar, ScreenX, ScreenY, Scrollbars, Scroll, ScrollBy,
            ScreenTop, ScreenLeft,
            ScrollTo, ScrollX, ScrollY, MoveBy, MoveTo, ResizeBy, ResizeTo, Self, _Window, Top, _Screen,
@@ -125,8 +140,9 @@ namespace KJS {
            ReleaseEvents, AddEventListener, RemoveEventListener, XMLHttpRequest, XMLSerializer,
 	   Onabort, Onblur, Onchange, Onclick, Ondblclick, Ondragdrop, Onerror, 
 	   Onfocus, Onkeydown, Onkeypress, Onkeyup, Onload, Onmousedown, Onmousemove,
-           Onmouseout, Onmouseover, Onmouseup, Onmove, Onreset, Onresize, Onscroll, Onsearch,
-           Onselect, Onsubmit, Onunload };
+           Onmouseout, Onmouseover, Onmouseup, OnWindowMouseWheel, Onmove, Onreset, Onresize, Onscroll, Onsearch,
+           Onselect, Onsubmit, Onunload,
+           Statusbar, Toolbar };
   protected:
     Value getListener(ExecState *exec, int eventId) const;
     void setListener(ExecState *exec, int eventId, Value func);
@@ -139,6 +155,12 @@ namespace KJS {
     FrameArray *frames;
     Location *loc;
     Selection *m_selection;
+    BarInfo *m_locationbar;
+    BarInfo *m_menubar;
+    BarInfo *m_personalbar;
+    BarInfo *m_scrollbars;
+    BarInfo *m_statusbar;
+    BarInfo *m_toolbar;
     WindowQObject *winq;
     DOM::Event *m_evt;
   };
@@ -225,6 +247,23 @@ namespace KJS {
     QGuardedPtr<KHTMLPart> m_part;
   };
 
+  class BarInfo : public ObjectImp {
+  public:
+    ~BarInfo();
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
+    enum { Visible };
+    enum Type { Locationbar, Menubar, Personalbar, Scrollbars, Statusbar, Toolbar };
+    KHTMLPart *part() const { return m_part; }
+    virtual const ClassInfo* classInfo() const { return &info; }
+    static const ClassInfo info;
+  private:
+    friend class Window;
+    BarInfo(ExecState *exec, KHTMLPart *p, Type barType);
+    QGuardedPtr<KHTMLPart> m_part;
+    Type m_type;
+  };
+
 #ifdef Q_WS_QWS
   class Konqueror : public ObjectImp {
     friend class KonquerorFunc;
