@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (C) 2004 Apple Computer, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -91,9 +91,12 @@ public:
 
     void ensureVisible(int,int);
     void ensureVisible(int,int,int,int);
-    void ensureRectVisibleCentered(const QRect &r);
+    void ensureRectVisibleCentered(const QRect &r, bool forceCentering=false);
         
     NSView *getDocumentView() const;
+
+private:
+    virtual bool isQScrollView() const;
 };
 
 #endif
